@@ -1,7 +1,7 @@
 /*
  * Copyright 2004-2008 J. Dahl and L. Vandenberghe.
  *
- * This file is part of CVXOPT version 0.9.3.
+ * This file is part of CVXOPT version 1.0.
  *
  * CVXOPT is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -65,7 +65,7 @@ static int get_param_idx(char *str, int *idx)
 
 static int set_defaults(double *control)
 {
-    int_compat pos=0; 
+    int_t pos=0; 
     int param_id; 
     PyObject *param, *key, *value;
     char *keystr, err_str[100];
