@@ -1,7 +1,7 @@
 /*
  * Copyright 2004-2008 J. Dahl and L. Vandenberghe.
  *
- * This file is part of CVXOPT version 0.9.3.
+ * This file is part of CVXOPT version 1.0.
  *
  * CVXOPT is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -69,7 +69,7 @@ static cholmod_common Common;
 
 static int set_options(void)
 {
-    int_compat pos=0;
+    int_t pos=0;
     PyObject *param, *key, *value;
     char *keystr, err_str[100];
 
