@@ -1,7 +1,7 @@
 /*
  * Copyright 2004-2008 J. Dahl and L. Vandenberghe.
  *
- * This file is part of CVXOPT version 0.9.3.
+ * This file is part of CVXOPT version 1.0.
  *
  * CVXOPT is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -28,23 +28,26 @@
 #define MAT_BUFZ(O)  ((complex *)((matrix *)O)->buffer)
 #endif
 
-
 #ifndef __CVXOPT__
 #define __CVXOPT__
 
+/*
+static void * malloc_aligned(size_t size) {
+  void *p = NULL;
+  if (!posix_memalign(&p, 16, size)) 
+    return p;
+  else 
+    return NULL;      
+} 
+
+#define malloc(size) malloc_aligned(size)
+*/
+
 #define INT       0
 #define DOUBLE    1
 #define COMPLEX   2
 
-/* compatibility between Python2.5 and lower versions */
-#if PY_VERSION_HEX < 0x02050000 
-#define int_t      Py_intptr_t
-#define lenfunc    inquiry
-#define int_compat int
-#else
 #define int_t      Py_ssize_t
-#define int_compat Py_ssize_t
-#endif
 
 typedef struct {
   PyObject_HEAD
