@@ -1,7 +1,7 @@
 /*
  * Copyright 2004-2008 J. Dahl and L. Vandenberghe.
  *
- * This file is part of CVXOPT version 0.9.3.
+ * This file is part of CVXOPT version 1.0.
  *
  * CVXOPT is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,7 +25,6 @@
 
 #include <complexobject.h>
 
-
 /* NumPy array protocol */
 typedef struct {
      int version;       
@@ -575,7 +574,7 @@ matrix * create_indexlist(int dim, PyObject *A)
   }
   /* slice */
   else if (PySlice_Check(A)) {
-    int_compat start, stop, step, lgt;
+    int_t start, stop, step, lgt;
 
     if (PySlice_GetIndicesEx((PySliceObject*)A, dim,
 	    &start, &stop, &step, &lgt) < 0) return NULL;
@@ -656,8 +655,8 @@ matrix_subscr(matrix* self, PyObject* args)
 
   /* two slices, handled separately for speed */
   if (PySlice_Check(argI) && PySlice_Check(argJ)) {
-    int_compat rowstart, rowstop, rowstep, rowlgt;
-    int_compat colstart, colstop, colstep, collgt;
+    int_t rowstart, rowstop, rowstep, rowlgt;
+    int_t colstart, colstop, colstep, collgt;
 
     if ( (PySlice_GetIndicesEx((PySliceObject*)argI, MAT_NROWS(self),
 		&rowstart, &rowstop, &rowstep, &rowlgt) < 0) ||	
@@ -793,8 +792,8 @@ matrix_ass_subscr(matrix* self, PyObject* args, PyObject* val)
   if (PySlice_Check(argI) && PySlice_Check(argJ) && 
       Matrix_Check(val) && MAT_ID(val) == MAT_ID(self)) {
 
-    int_compat rowstart, rowstop, rowstep, rowlgt;
-    int_compat colstart, colstop, colstep, collgt;
+    int_t rowstart, rowstop, rowstep, rowlgt;
+    int_t colstart, colstop, colstep, collgt;
 
     if ( (PySlice_GetIndicesEx((PySliceObject*)argI, MAT_NROWS(self),
 		&rowstart, &rowstop, &rowstep, &rowlgt) < 0) ||	
@@ -1067,6 +1066,34 @@ matrix_get_size(matrix *self, void *closure)
   return t;
 }
 
+static int 
+matrix_set_size(matrix *self, PyObject *value, void *closure)
+{
+  if (value == NULL) 
+    PY_ERR_INT(PyExc_TypeError, "size attribute cannot be deleted");
+
+  if (!PyTuple_Check(value) || PyTuple_Size(value) != 2)
+    PY_ERR_INT(PyExc_TypeError, "can only assign a 2-tuple to size");
+
+  if (!PyInt_Check(PyTuple_GET_ITEM(value, 0)) || 
+      !PyInt_Check(PyTuple_GET_ITEM(value, 1)))
+    PY_ERR_INT(PyExc_TypeError, "invalid size tuple");
+    
+  int m = PyInt_AS_LONG(PyTuple_GET_ITEM(value, 0));
+  int n = PyInt_AS_LONG(PyTuple_GET_ITEM(value, 1));
+
+  if (m<0 || n<0)
+    PY_ERR_INT(PyExc_TypeError, "dimensions must be non-negative");
+  
+  if (m*n != MAT_LGT(self))
+    PY_ERR_INT(PyExc_TypeError, "number of elements in matrix cannot change");
+
+  MAT_NROWS(self) = m;
+  MAT_NCOLS(self) = n;
+
+  return 0;
+}
+
 static PyObject * matrix_get_typecode(matrix *self, void *closure)
 {
   return PyString_FromStringAndSize(TC_CHAR[self->id], 1);
@@ -1119,7 +1146,8 @@ static PyObject * matrix_get_H(matrix *self, void *closure)
 }
 
 static PyGetSetDef matrix_getsets[] = {
-  {"size", (getter) matrix_get_size, NULL, "matrix dimensions"}, 
+  {"size", (getter) matrix_get_size, (setter) matrix_set_size, 
+   "matrix dimensions"}, 
   {"typecode", (getter) matrix_get_typecode, NULL, "typecode character"},
   {"__array_struct__", (getter) matrix_array_struct, NULL,
    "C object implementing the NumPy array protocol"},
