@@ -1,7 +1,7 @@
 /*
  * Copyright 2004-2008 J. Dahl and L. Vandenberghe.
  *
- * This file is part of CVXOPT version 0.9.3.
+ * This file is part of CVXOPT version 1.0.
  *
  * CVXOPT is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -110,7 +110,7 @@ static PyObject* solvesdp(PyObject *self, PyObject *args,
         *param, *key, *value;
     int i, j, k, n, ml, l, mk, nnz, *lp_colptr=NULL, *lp_rowind=NULL,
       incx, incy, lngth, maxm;
-    int_compat pos=0;
+    int_t pos=0;
     double *lp_values=NULL, *zlvals=NULL, *zk=NULL, r, beta=-1.0, 
         gamma=-1.0, tol;
     dsdp_matrix **lmis=NULL;
