@@ -8,7 +8,7 @@ to the quadratic programming solver from MOSEK.
 
 # Copyright 2004-2008 J. Dahl and L. Vandenberghe.
 # 
-# This file is part of CVXOPT version 0.9.3.
+# This file is part of CVXOPT version 1.0.
 #
 # CVXOPT is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -318,6 +318,9 @@ def cpl(c, F, G = None, h = None, dims = None, A = None, b = None,
     EXPON = 3
     MAX_RELAXED_ITERS = 8
 
+    try: DEBUG = options['debug']
+    except KeyError: DEBUG = False
+
     try: MAXITERS = options['maxiters']
     except KeyError: MAXITERS = 100
     else: 
@@ -548,11 +551,10 @@ def cpl(c, F, G = None, h = None, dims = None, A = None, b = None,
         print "% 10s% 12s% 10s% 8s% 7s" %("pcost", "dcost", "gap", "pres",
             "dres")
 
-
     relaxed_iters = 0
     for iters in xrange(MAXITERS):  
 
-        if refinement:  
+        if refinement or DEBUG:  
             # We need H to compute residuals of KKT equations.
             f, Df, H = F(x, z[:mnl])
         else:
@@ -578,7 +580,7 @@ def cpl(c, F, G = None, h = None, dims = None, A = None, b = None,
                     "a user-provided kktsolver"
             fDf = Df
 
-        if refinement:
+        if refinement or DEBUG:
             if type(H) is matrix or type(H) is spmatrix:
                 if customx: raise ValueError, "use of non-vector type "\
                     "for  x requires function valued H"
@@ -811,7 +813,7 @@ def cpl(c, F, G = None, h = None, dims = None, A = None, b = None,
         # iterative refinement.
 
         if iters == 0:
-            if refinement:
+            if refinement or DEBUG:
                 wx, wy = xnewcopy(c), ynewcopy(b)
                 wz = matrix(0.0, (mnl + cdim, 1))
                 ws = matrix(0.0, (mnl + cdim, 1))
@@ -821,7 +823,7 @@ def cpl(c, F, G = None, h = None, dims = None, A = None, b = None,
                 ws2 = matrix(0.0, (mnl + cdim, 1))
 
         def f4(x, y, z, s):
-            if refinement: 
+            if refinement or DEBUG: 
                 xcopy(x, wx)        
                 ycopy(y, wy)        
                 blas.copy(z, wz)        
@@ -838,6 +840,14 @@ def cpl(c, F, G = None, h = None, dims = None, A = None, b = None,
                 yaxpy(wy2, y)
                 blas.axpy(wz2, z)
                 blas.axpy(ws2, s)
+            if DEBUG:
+                res(x, y, z, s, wx, wy, wz, ws)
+                print "KKT residuals:"
+                print "    'x': %e" %math.sqrt(xdot(wx, wx)) 
+                print "    'y': %e" %math.sqrt(ydot(wy, wy))
+                print "    'z': %e" %misc.snrm2(wz, dims, mnl)
+                print "    's': %e" %misc.snrm2(ws, dims, mnl)
+     
 
         sigma, eta = 0.0, 0.0
         for i in [0, 1]:
