@@ -1,6 +1,6 @@
 # Copyright 2004-2008 J. Dahl and L. Vandenberghe.
 # 
-# This file is part of CVXOPT version 0.9.3.
+# This file is part of CVXOPT version 1.0.
 #
 # CVXOPT is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,36 +16,43 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 import math
-from cvxopt import base, blas, lapack, cholmod
+from cvxopt import base, blas, lapack, cholmod, misc_solvers
 from cvxopt.base import matrix, spmatrix
 __all__ = []
 
+use_C = True
 
-def scale(x, W, trans = 'N', inverse = 'N'):  
-
-    # Computes 
-    #
-    #     x := W*x        (trans is 'N', inverse = 'N')  
-    #     x := W^T*x      (trans is 'T', inverse = 'N')  
-    #     x := W^{-1}*x   (trans is 'N', inverse = 'I')  
-    #     x := W^{-T}*x   (trans is 'T', inverse = 'I'). 
-    #
-    # x is a dense 'd' matrix.
-    #
-    # W is a dictionary with entries:
-    #
-    # - W['dnl']: positive vector
-    # - W['dnli']: componentwise inverse of W['dnl']
-    # - W['d']: positive vector
-    # - W['di']: componentwise inverse of W['d']
-    # - W['v']: lists of 2nd order cone vectors with unit hyperbolic norms
-    # - W['beta']: list of positive numbers
-    # - W['r']: list of square matrices 
-    # - W['rti']: list of square matrices.  rti[k] is the inverse transpose
-    #   of r[k].
-    #
-    # The 'dnl' and 'dnli' entries are optional, and only present when the 
-    # function is called from the nonlinear solver.
+if use_C:
+  scale = misc_solvers.scale
+else:
+  def scale(x, W, trans = 'N', inverse = 'N'):  
+    """
+    Applies Nesterov-Todd scaling or its inverse.
+    
+    Computes 
+    
+         x := W*x        (trans is 'N', inverse = 'N')  
+         x := W^T*x      (trans is 'T', inverse = 'N')  
+         x := W^{-1}*x   (trans is 'N', inverse = 'I')  
+         x := W^{-T}*x   (trans is 'T', inverse = 'I'). 
+    
+    x is a dense 'd' matrix.
+    
+    W is a dictionary with entries:
+    
+    - W['dnl']: positive vector
+    - W['dnli']: componentwise inverse of W['dnl']
+    - W['d']: positive vector
+    - W['di']: componentwise inverse of W['d']
+    - W['v']: lists of 2nd order cone vectors with unit hyperbolic norms
+    - W['beta']: list of positive numbers
+    - W['r']: list of square matrices 
+    - W['rti']: list of square matrices.  rti[k] is the inverse transpose
+      of r[k].
+    
+    The 'dnl' and 'dnli' entries are optional, and only present when the 
+    function is called from the nonlinear solver.
+    """
 
     ind = 0
 
@@ -155,12 +162,18 @@ def scale(x, W, trans = 'N', inverse = 'N'):
         ind += n**2
 
 
-def scale2(lmbda, x, dims, mnl = 0, inverse = 'N'):
+if use_C:
+  scale2 = misc_solvers.scale2
+else:
+  def scale2(lmbda, x, dims, mnl = 0, inverse = 'N'):
+    """
+    Evaluates
 
-    # x := H(lambda^{1/2}) * x   (inverse is 'N')
-    # x := H(lambda^{-1/2}) * x  (inverse is 'I')
-    #
-    # H is the Hessian of the logarithmic barrier.
+        x := H(lambda^{1/2}) * x   (inverse is 'N')
+        x := H(lambda^{-1/2}) * x  (inverse is 'I').
+    
+    H is the Hessian of the logarithmic barrier.
+    """
       
 
     # For the nonlinear and 'l' blocks, 
@@ -233,11 +246,13 @@ def scale2(lmbda, x, dims, mnl = 0, inverse = 'N'):
 
 
 def compute_scaling(s, z, lmbda, dims, mnl = None):
+    """
+    Returns the Nesterov-Todd scaling W at points s and z, and stores the 
+    scaled variable in lmbda. 
+    
+        W * z = W^{-T} * s = lmbda. 
 
-    # Returns the Nesterov-Todd scaling W at points s and z, and stores
-    # the scaled variable in lmbda. 
-    #
-    #    W * z = W^{-T} * s = lmbda. 
+    """
      
     W = {}
 
@@ -403,19 +418,20 @@ def compute_scaling(s, z, lmbda, dims, mnl = None):
 
 
 def update_scaling(W, lmbda, s, z):
-
-    # Updates the Nesterov-Todd scaling matrix W and the scaled variable 
-    # lmbda so that on exit
-    #
-    #      W * zt = W^{-T} * st = lmbda.
-    #
-    # On entry, the nonlinear, 'l' and 'q' components of the arguments s 
-    # and z contain W^{-T}*st and W*zt, i.e, the new iterates in the 
-    # current scaling.
-    #
-    # The 's' components contain the factors Ls, Lz in a factorization of 
-    # the new iterates in the current scaling, W^{-T}*st = Ls*Ls',   
-    # W*zt = Lz*Lz'.
+    """
+    Updates the Nesterov-Todd scaling matrix W and the scaled variable 
+    lmbda so that on exit
+    
+          W * zt = W^{-T} * st = lmbda.
+     
+    On entry, the nonlinear, 'l' and 'q' components of the arguments s 
+    and z contain W^{-T}*st and W*zt, i.e, the new iterates in the current 
+    scaling.
+    
+    The 's' components contain the factors Ls, Lz in a factorization of 
+    the new iterates in the current scaling, W^{-T}*st = Ls*Ls',   
+    W*zt = Lz*Lz'.
+    """
   
 
     # Nonlinear and 'l' blocks
@@ -616,15 +632,20 @@ def update_scaling(W, lmbda, s, z):
         ind3 += m
 
 
-def pack(x, y, dims, mnl = 0, offsetx = 0, offsety = 0):
-
-     # The vector x is an element of S, with the 's' components stored 
-     # in unpacked storage.  On return, x is copied to y with the 's' 
-     # components stored in packed storage and the off-diagonal entries 
-     # scaled by sqrt(2).
+if use_C:
+  pack = misc_solvers.pack
+else:
+  def pack(x, y, dims, mnl = 0, offsetx = 0, offsety = 0):
+     """
+     Copy x to y using packed storage.
+    
+     The vector x is an element of S, with the 's' components stored in 
+     unpacked storage.  On return, x is copied to y with the 's' components
+     stored in packed storage and the off-diagonal entries scaled by 
+     sqrt(2).
+     """
 
      nlq = mnl + dims['l'] + sum(dims['q'])
-     np = sum([ n*(n+1)/2 for n in dims['s'] ])
      blas.copy(x, y, n = nlq, offsetx = offsetx, offsety = offsety)
      iu, ip = offsetx + nlq, offsety + nlq
      for n in dims['s']:
@@ -633,18 +654,46 @@ def pack(x, y, dims, mnl = 0, offsetx = 0, offsety = 0):
              y[ip] /= math.sqrt(2)
              ip += n-k
          iu += n**2 
+     np = sum([ n*(n+1)/2 for n in dims['s'] ])
      blas.scal(math.sqrt(2.0), y, n = np, offset = offsety+nlq)
      
 
-def unpack(x, y, dims, mnl = 0, offsetx = 0, offsety = 0):
+if use_C:
+  pack2 = misc_solvers.pack2
+else:
+  def pack2(x, dims, mnl = 0):
+     """
+     In-place version of pack(), which also accepts matrix arguments x.  
+     The columns of x are elements of S, with the 's' components stored in
+     unpacked storage.  On return, the 's' components are stored in packed
+     storage and the off-diagonal entries are scaled by sqrt(2).
+     """
 
-     # The vector x is an element of S, with the 's' components stored
-     # in unpacked storage and off-diagonal entries scaled by sqrt(2).
-     # On return, x is copied to y with the 's' components stored in 
-     # unpacked storage and off-diagonal entries scaled by sqrt(2).
+     if not dims['s']: return
+     iu = mnl + dims['l'] + sum(dims['q'])
+     ip = iu
+     for n in dims['s']:
+         for k in xrange(n):
+             x[ip, :] = x[iu + (n+1)*k, :]
+             x[ip + 1 : ip+n-k, :] = x[iu + (n+1)*k + 1: iu + n*(k+1), :] \
+                 * math.sqrt(2.0)
+             ip += n - k
+         iu += n**2 
+     np = sum([ n*(n+1)/2 for n in dims['s'] ])
+     
+
+if use_C:
+  unpack = misc_solvers.unpack
+else:
+  def unpack(x, y, dims, mnl = 0, offsetx = 0, offsety = 0):
+     """
+     The vector x is an element of S, with the 's' components stored
+     in unpacked storage and off-diagonal entries scaled by sqrt(2).
+     On return, x is copied to y with the 's' components stored in 
+     unpacked storage.
+     """
 
      nlq = mnl + dims['l'] + sum(dims['q'])
-     nu = sum([ n**2 for n in dims['s'] ])
      blas.copy(x, y, n = nlq, offsetx = offsetx, offsety = offsety)
      iu, ip = offsety+nlq, offsetx+nlq
      for n in dims['s']:
@@ -653,12 +702,17 @@ def unpack(x, y, dims, mnl = 0, offsetx = 0, offsety = 0):
              y[iu+k*(n+1)] *= math.sqrt(2)
              ip += n-k
          iu += n**2 
+     nu = sum([ n**2 for n in dims['s'] ])
      blas.scal(1.0/math.sqrt(2.0), y, n = nu, offset = offsety+nlq)
 
 
-def sdot(x, y, dims, mnl = 0):
-
-    # Returns the inner product of two vectors in S.
+if use_C:
+  sdot = misc_solvers.sdot
+else:
+  def sdot(x, y, dims, mnl = 0):
+    """
+    Inner product of two vectors in S.
+    """
     
     ind = mnl + dims['l'] + sum(dims['q'])
     a = blas.dot(x, y, n = ind)
@@ -699,85 +753,116 @@ def sdot2(x, y):
 
 
 def snrm2(x, dims, mnl = 0): 
-
-    # Returns the norm of a vector in S
+    """
+    Returns the norm of a vector in S
+    """
 
     return math.sqrt(sdot(x, x, dims, mnl))
 
 
-def sgemv(A, x, y, dims, trans = 'N', alpha = 1.0, beta = 0.0, m = None, 
-    n = None, offsetA = 0, offsetx = 0, offsety = 0): 
+if use_C:
+  trisc = misc_solvers.trisc
+else:
+  def trisc(x, dims, offset = 0): 
+    """
+    Sets upper triangular part of the 's' components of x equal to zero
+    and scales the strictly lower triangular part by 2.0.
+    """ 
 
-    # A is a matrix or spmatrix of size (N, n) where 
-    #
-    #     N = dims['l'] + sum(dims['q']) + sum( k**2 for k in dims['s'] ). 
-    #
-    # If trans is 'N': 
-    #
-    #     y := alpha*A*x + beta * y   (trans = 'N').
-    #
-    # x is a vector of length n.  y is a vector of length N.
-    #
-    # If trans is 'T':
-    #
-    #     y := alpha*A'*x + beta * y  (trans = 'T').
-    #
-    # x is a vector of length N.  y is a vector of length n.
-    #
-    # The 's' components in S are stored in unpacked 'L' storage.
-
-    if m is None: m = A.size[0]
-    if n is None: n = A.size[1]
-
-    if trans == 'T' and alpha:
-        ind = offsetx + dims['l'] + sum(dims['q'])
-        for mk in dims['s']:
-            # Set upper triangular part of x to zero and scale strict 
-            # lower triangular part by 2.
-            for j in xrange(1, mk):  
-                blas.scal(0.0, x, n = mk-j, inc = mk, offset = 
+    m = dims['l'] + sum(dims['q']) + sum([ k**2 for k in dims['s'] ]) 
+    ind = offset + dims['l'] + sum(dims['q'])
+    for mk in dims['s']:
+        for j in xrange(1, mk):  
+            blas.scal(0.0, x, n = mk-j, inc = mk, offset = 
                     ind + j*(mk + 1) - 1) 
-                blas.scal(2.0, x, offset = ind + mk*(j-1) + j, n = mk-j) 
-            ind += mk**2
+            blas.scal(2.0, x, offset = ind + mk*(j-1) + j, n = mk-j) 
+        ind += mk**2
 
+
+if use_C:
+  triusc = misc_solvers.triusc
+else:
+  def triusc(x, dims, offset = 0):
+    """
+    Scales the strictly lower triangular part of the 's' components of x 
+    by 0.5.
+    """ 
+
+    m = dims['l'] + sum(dims['q']) + sum([ k**2 for k in dims['s'] ]) 
+    ind = offset + dims['l'] + sum(dims['q'])
+    for mk in dims['s']:
+        for j in xrange(1, mk):  
+                blas.scal(0.5, x, offset = ind + mk*(j-1) + j, n = mk-j) 
+        ind += mk**2
+
+
+def sgemv(A, x, y, dims, trans = 'N', alpha = 1.0, beta = 0.0, n = None, 
+    offsetA = 0, offsetx = 0, offsety = 0): 
+    """
+    Matrix-vector multiplication.
+
+    A is a matrix or spmatrix of size (m, n) where 
+    
+        N = dims['l'] + sum(dims['q']) + sum( k**2 for k in dims['s'] ) 
+
+    representing a method from R^n to S.  
+    
+    If trans is 'N': 
+    
+        y := alpha*A*x + beta * y   (trans = 'N').
+    
+    x is a vector of length n.  y is a vector of length N.
+    
+    If trans is 'T':
+    
+        y := alpha*A'*x + beta * y  (trans = 'T').
+    
+    x is a vector of length N.  y is a vector of length n.
+    
+    The 's' components in S are stored in unpacked 'L' storage.
+    """
+
+    m = dims['l'] + sum(dims['q']) + sum([ k**2 for k in dims['s'] ]) 
+    if n is None: n = A.size[1]
+    if trans == 'T' and alpha:  trisc(x, dims, offsetx)
     base.gemv(A, x, y, trans = trans, alpha = alpha, beta = beta, m = m,
         n = n, offsetA = offsetA, offsetx = offsetx, offsety = offsety)
-
-    if trans == 'T' and alpha:
-        ind = offsetx + dims['l'] + sum(dims['q'])
-        for mk in dims['s']:
-            # Scale strict lower triangular part of x by 0.5.
-            for j in xrange(1, mk):  
-                blas.scal(0.5, x, offset = ind + mk*(j-1) + j, n = mk-j) 
-            ind += mk**2
+    if trans == 'T' and alpha: triusc(x, dims, offsetx)
 
 
 def jdot(x, y, n = None, offsetx = 0, offsety = 0):
-
-    # Returns x' * J * y, where J = [1, 0; 0, -I].
+    """
+    Returns x' * J * y, where J = [1, 0; 0, -I].
+    """
 
     if n is None: 
-         if len(x) != len(y): raise ValueError, "x and y must have the "\
+        if len(x) != len(y): raise ValueError, "x and y must have the "\
              "same length"
-         n = len(x)
+        n = len(x)
     return x[offsetx] * y[offsety] - blas.dot(x, y, n = n-1, 
         offsetx = offsetx + 1, offsety = offsety + 1) 
 
 
 def jnrm2(x, n = None, offset = 0):
-
-    # Returns sqrt(x' * J * x) where J = [1, 0; 0, -I], for a vector
-    # x in a second order cone. 
+    """
+    Returns sqrt(x' * J * x) where J = [1, 0; 0, -I], for a vector
+    x in a second order cone. 
+    """
 
     if n is None:  n = len(x)
     a = blas.nrm2(x, n = n-1, offset = offset+1)
     return math.sqrt(x[offset] - a) * math.sqrt(x[offset] + a)
 
 
-def symm(x, n, offset = 0):
-
-    # Fills in the upper triangular part of the symmetric matrix stored in
-    # x[offset : offset+n*n] using 'L' storage.
+if use_C:
+  symm = misc_solvers.symm
+else:
+  def symm(x, n, offset = 0):
+    """
+    Converts lower triangular matrix to symmetric.  
+    Fills in the upper triangular part of the symmetric matrix stored in 
+    x[offset : offset+n*n] using 'L' storage.
+    """
 
     if n <= 1:  return
     for i in xrange(n-1):
@@ -785,10 +870,14 @@ def symm(x, n, offset = 0):
             offset + (i+1)*(n+1) - 1, incy = n, n = n-i-1)
 
 
-def sprod(x, y, dims, mnl = 0, diag = 'N'):   
-
-    # The product x := (y o x).  If diag is 'D', the 's' part of y is 
-    # diagonal and only the diagonal is stored.
+if use_C:
+  sprod = misc_solvers.sprod
+else:
+  def sprod(x, y, dims, mnl = 0, diag = 'N'):   
+    """
+    The product x := (y o x).  If diag is 'D', the 's' part of y is 
+    diagonal and only the diagonal is stored.
+    """
 
 
     # For the nonlinear and 'l' blocks:  
@@ -800,9 +889,9 @@ def sprod(x, y, dims, mnl = 0, diag = 'N'):
 
     # For 'q' blocks: 
     #
-    #               [ lo   l1'  ]
+    #               [ l0   l1'  ]
     #     yk o xk = [           ] * xk
-    #               [ lo   l0*I ] 
+    #               [ l1   l0*I ] 
     #
     # where yk = (l0, l1).
     
@@ -852,9 +941,10 @@ def sprod(x, y, dims, mnl = 0, diag = 'N'):
 
 
 def ssqr(x, y, dims, mnl = 0):
-
-    # The product x := y o y.   The 's' components of y are diagonal and
-    # only the diagonals of x and y are stored.     
+    """
+    The product x := y o y.   The 's' components of y are diagonal and
+    only the diagonals of x and y are stored.     
+    """
 
     blas.copy(y, x)
     blas.tbmv(y, x, n = mnl + dims['l'], k = 0, ldA = 1) 
@@ -867,10 +957,14 @@ def ssqr(x, y, dims, mnl = 0):
         offsetx = ind) 
 
 
-def sinv(x, y, dims, mnl = 0):   
-
-    # The inverse product x := (y o\ x), when the 's' components of y are 
-    # diagonal.
+if use_C:
+  sinv = misc_solvers.sinv
+else:
+  def sinv(x, y, dims, mnl = 0):   
+    """
+    The inverse product x := (y o\ x), when the 's' components of y are 
+    diagonal.
+    """
     
     # For the nonlinear and 'l' blocks:  
     # 
@@ -916,16 +1010,20 @@ def sinv(x, y, dims, mnl = 0):
         ind2 += m
 
 
-def max_step(x, dims, mnl = 0, sigma = None):
-
-    # Returns min {t | x + t*e >= 0}, where e is defined as follows
-    #
-    # - For the nonlinear and 'l' blocks: e is the vector of ones.
-    # - For the 'q' blocks: e is the first unit vector.
-    # - For the 's' blocks: e is the identity matrix.
-    #
-    # When called with the argument sigma, also returns the eigenvalues 
-    # (in sigma) and the eigenvectors (in x) of the 's' components of x.
+if use_C:
+  max_step = misc_solvers.max_step
+else:
+  def max_step(x, dims, mnl = 0, sigma = None):
+    """
+    Returns min {t | x + t*e >= 0}, where e is defined as follows
+    
+    - For the nonlinear and 'l' blocks: e is the vector of ones.
+    - For the 'q' blocks: e is the first unit vector.
+    - For the 's' blocks: e is the identity matrix.
+    
+    When called with the argument sigma, also returns the eigenvalues 
+    (in sigma) and the eigenvectors (in x) of the 's' components of x.
+    """
 
     t = []
     ind = mnl + dims['l']
@@ -953,25 +1051,26 @@ def max_step(x, dims, mnl = 0, sigma = None):
 
 
 def kkt_ldl(G, dims, A, mnl = 0):
-
-    # Solution of KKT equations by a dense LDL factorization of the 
-    # 3 x 3 system.
-    #
-    # Returns a function that (1) computes the LDL factorization of
-    #
-    #     [ H           A'   GG'*W^{-1} ] 
-    #     [ A           0    0          ],
-    #     [ W^{-T}*GG   0   -I          ] 
-    #
-    # given H, Df, W, where GG = [Df; G], and (2) returns a function for 
-    # solving 
-    #
-    #     [ H     A'   GG'   ]   [ ux ]   [ bx ]
-    #     [ A     0    0     ] * [ uy ] = [ by ].
-    #     [ GG    0   -W'*W  ]   [ uz ]   [ bz ]
-    #
-    # H is n x n,  A is p x n, Df is mnl x n, G is N x n where
-    # N = dims['l'] + sum(dims['q']) + sum( k**2 for k in dims['s'] ).
+    """
+    Solution of KKT equations by a dense LDL factorization of the 
+    3 x 3 system.
+    
+    Returns a function that (1) computes the LDL factorization of
+    
+        [ H           A'   GG'*W^{-1} ] 
+        [ A           0    0          ],
+        [ W^{-T}*GG   0   -I          ] 
+    
+    given H, Df, W, where GG = [Df; G], and (2) returns a function for 
+    solving 
+    
+        [ H     A'   GG'   ]   [ ux ]   [ bx ]
+        [ A     0    0     ] * [ uy ] = [ by ].
+        [ GG    0   -W'*W  ]   [ uz ]   [ bz ]
+    
+    H is n x n,  A is p x n, Df is mnl x n, G is N x n where
+    N = dims['l'] + sum(dims['q']) + sum( k**2 for k in dims['s'] ).
+    """
     
     p, n = A.size
     ldK = n + p + mnl + dims['l'] + sum(dims['q']) + sum([ k*(k+1)/2 for k 
@@ -1022,25 +1121,26 @@ def kkt_ldl(G, dims, A, mnl = 0):
 
 
 def kkt_ldl2(G, dims, A, mnl = 0):
-
-    # Solution of KKT equations by a dense LDL factorization of the 2 x 2 
-    # system.
-    #
-    # Returns a function that (1) computes the LDL factorization of
-    #
-    #     [ H + GG' * W^{-1} * W^{-T} * GG   A' ]
-    #     [                                     ]
-    #     [ A                                0  ]
-    #
-    # given H, Df, W, where GG = [Df; G], and (2) returns a function for 
-    # solving 
-    #
-    #     [ H    A'   GG'   ]   [ ux ]   [ bx ]
-    #     [ A    0    0     ] * [ uy ] = [ by ].
-    #     [ GG   0   -W'*W  ]   [ uz ]   [ bz ]
-    #
-    # H is n x n,  A is p x n, Df is mnl x n, G is N x n where
-    # N = dims['l'] + sum(dims['q']) + sum( k**2 for k in dims['s'] ).
+    """
+    Solution of KKT equations by a dense LDL factorization of the 2 x 2 
+    system.
+    
+    Returns a function that (1) computes the LDL factorization of
+    
+        [ H + GG' * W^{-1} * W^{-T} * GG   A' ]
+        [                                     ]
+        [ A                                0  ]
+    
+    given H, Df, W, where GG = [Df; G], and (2) returns a function for 
+    solving 
+    
+        [ H    A'   GG'   ]   [ ux ]   [ bx ]
+        [ A    0    0     ] * [ uy ] = [ by ].
+        [ GG   0   -W'*W  ]   [ uz ]   [ bz ]
+    
+    H is n x n,  A is p x n, Df is mnl x n, G is N x n where
+    N = dims['l'] + sum(dims['q']) + sum( k**2 for k in dims['s'] ).
+    """
 
     p, n = A.size
     ldK = n + p 
@@ -1106,28 +1206,29 @@ def kkt_ldl2(G, dims, A, mnl = 0):
 
 
 def kkt_chol(G, dims, A, mnl = 0):
-
-    # Solution of KKT equations by reduction to a 2 x 2 system, a QR 
-    # factorization to eliminate the equality constraints, and a dense 
-    # Cholesky factorization of order n-p. 
-    #
-    # Computes the QR factorization
-    #
-    #     A' = [Q1, Q2] * [R; 0]
-    #
-    # and returns a function that (1) computes the Cholesky factorization 
-    #
-    #     Q_2^T * (H + GG^T * W^{-1} * W^{-T} * GG) * Q2 = L * L^T, 
-    #
-    # given H, Df, W, where GG = [Df; G], and (2) returns a function for 
-    # solving 
-    #
-    #     [ H    A'   GG'    ]   [ ux ]   [ bx ]
-    #     [ A    0    0      ] * [ uy ] = [ by ].
-    #     [ GG   0    -W'*W  ]   [ uz ]   [ bz ]
-    #
-    # H is n x n,  A is p x n, Df is mnl x n, G is N x n where
-    # N = dims['l'] + sum(dims['q']) + sum( k**2 for k in dims['s'] ).
+    """
+    Solution of KKT equations by reduction to a 2 x 2 system, a QR 
+    factorization to eliminate the equality constraints, and a dense 
+    Cholesky factorization of order n-p. 
+    
+    Computes the QR factorization
+    
+        A' = [Q1, Q2] * [R; 0]
+    
+    and returns a function that (1) computes the Cholesky factorization 
+    
+        Q_2^T * (H + GG^T * W^{-1} * W^{-T} * GG) * Q2 = L * L^T, 
+    
+    given H, Df, W, where GG = [Df; G], and (2) returns a function for 
+    solving 
+    
+        [ H    A'   GG'    ]   [ ux ]   [ bx ]
+        [ A    0    0      ] * [ uy ] = [ by ].
+        [ GG   0    -W'*W  ]   [ uz ]   [ bz ]
+    
+    H is n x n,  A is p x n, Df is mnl x n, G is N x n where
+    N = dims['l'] + sum(dims['q']) + sum( k**2 for k in dims['s'] ).
+    """
 
     p, n = A.size
     cdim = mnl + dims['l'] + sum(dims['q']) + sum([ k**2 for k in 
@@ -1144,7 +1245,6 @@ def kkt_chol(G, dims, A, mnl = 0):
     lapack.geqrf(QA, tauA)
 
     Gs = matrix(0.0, (cdim, n))
-    g = matrix(0.0, (cdim, 1))
     K = matrix(0.0, (n,n)) 
     bzp = matrix(0.0, (cdim_pckd, 1))
     yy = matrix(0.0, (p,1))
@@ -1164,13 +1264,11 @@ def kkt_chol(G, dims, A, mnl = 0):
             Gs[:mnl, :] = Df
         Gs[mnl:, :] = G
         scale(Gs, W, trans = 'T', inverse = 'I')
-        for k in xrange(n):
-            g[:] = Gs[:, k]
-            pack(g, Gs, dims, mnl, offsety = k*Gs.size[0])
+        pack2(Gs, dims, mnl)
 
         # K = [Q1, Q2]' * (H + Gs' * Gs) * [Q1, Q2].
-        if H is not None: K[:,:] = H
-        blas.syrk(Gs, K, beta = 1.0, k = cdim_pckd, trans = 'T')
+        blas.syrk(Gs, K, k = cdim_pckd, trans = 'T')
+        if H is not None: K[:,:] += H
         symm(K, n)
         lapack.ormqr(QA, tauA, K, side = 'L', trans = 'T')
         lapack.ormqr(QA, tauA, K, side = 'R')
@@ -1247,32 +1345,33 @@ def kkt_chol(G, dims, A, mnl = 0):
 
 
 def kkt_chol2(G, dims, A, mnl = 0):
-
-    # Solution of KKT equations by reduction to a 2 x 2 system, a sparse 
-    # or dense Cholesky factorization of order n to eliminate the 1,1 
-    # block, and a sparse or dense Cholesky factorization of order p.
-    # Implemented only for problems with no second-order or semidefinite
-    # cone constraints.
-    #
-    # Returns a function that (1) computes Cholesky factorizations of
-    # the matrices 
-    #
-    #     S = H + GG' * W^{-1} * W^{-T} * GG,  
-    #     K = A * S^{-1} *A'
-    #
-    # or (if K is singular in the first call to the function), the matrices
-    #
-    #     S = H + GG' * W^{-1} * W^{-T} * GG + A' * A,  
-    #     K = A * S^{-1} * A',
-    #
-    # given H, Df, W, where GG = [Df; G], and (2) returns a function for 
-    # solving 
-    #
-    #     [ H     A'   GG'   ]   [ ux ]   [ bx ]
-    #     [ A     0    0     ] * [ uy ] = [ by ].
-    #     [ GG    0   -W'*W  ]   [ uz ]   [ bz ]
-    #
-    # H is n x n,  A is p x n, Df is mnl x n, G is dims['l'] x n.
+    """
+    Solution of KKT equations by reduction to a 2 x 2 system, a sparse 
+    or dense Cholesky factorization of order n to eliminate the 1,1 
+    block, and a sparse or dense Cholesky factorization of order p.
+    Implemented only for problems with no second-order or semidefinite
+    cone constraints.
+    
+    Returns a function that (1) computes Cholesky factorizations of
+    the matrices 
+    
+        S = H + GG' * W^{-1} * W^{-T} * GG,  
+        K = A * S^{-1} *A'
+    
+    or (if K is singular in the first call to the function), the matrices
+    
+        S = H + GG' * W^{-1} * W^{-T} * GG + A' * A,  
+        K = A * S^{-1} * A',
+    
+    given H, Df, W, where GG = [Df; G], and (2) returns a function for 
+    solving 
+    
+        [ H     A'   GG'   ]   [ ux ]   [ bx ]
+        [ A     0    0     ] * [ uy ] = [ by ].
+        [ GG    0   -W'*W  ]   [ uz ]   [ bz ]
+    
+    H is n x n,  A is p x n, Df is mnl x n, G is dims['l'] x n.
+    """
 
     if dims['q'] or dims['s']:
         raise ValueError, "kktsolver option 'kkt_chol2' is implemented "\
@@ -1464,28 +1563,29 @@ def kkt_chol2(G, dims, A, mnl = 0):
 
 
 def kkt_qr(G, dims, A):
-
-    # Solution of KKT equations with zero 1,1 block, by eliminating the
-    # equality constraints via a QR factorization, and solving the
-    # reduced KKT system by another QR factorization.
-    #
-    # Computes the QR factorization
-    #
-    #     A' = [Q1, Q2] * [R1; 0]
-    #
-    # and returns a function that (1) computes the QR factorization 
-    #
-    #     W^{-T} * G * Q2 = Q3 * R3
-    #
-    # (with columns of W^{-T}*G in packed storage), and (2) returns a 
-    # function for # solving 
-    #
-    #     [ 0    A'   G'    ]   [ ux ]   [ bx ]
-    #     [ A    0    0     ] * [ uy ] = [ by ].
-    #     [ G    0   -W'*W  ]   [ uz ]   [ bz ]
-    #
-    # A is p x n and G is N x n where N = dims['l'] + sum(dims['q']) + 
-    # sum( k**2 for k in dims['s'] ).
+    """
+    Solution of KKT equations with zero 1,1 block, by eliminating the
+    equality constraints via a QR factorization, and solving the
+    reduced KKT system by another QR factorization.
+    
+    Computes the QR factorization
+    
+        A' = [Q1, Q2] * [R1; 0]
+    
+    and returns a function that (1) computes the QR factorization 
+    
+        W^{-T} * G * Q2 = Q3 * R3
+    
+    (with columns of W^{-T}*G in packed storage), and (2) returns a 
+    function for # solving 
+    
+        [ 0    A'   G'    ]   [ ux ]   [ bx ]
+        [ A    0    0     ] * [ uy ] = [ by ].
+        [ G    0   -W'*W  ]   [ uz ]   [ bz ]
+    
+    A is p x n and G is N x n where N = dims['l'] + sum(dims['q']) + 
+    sum( k**2 for k in dims['s'] ).
+    """
  
     p, n = A.size
     cdim = dims['l'] + sum(dims['q']) + sum([ k**2 for k in dims['s'] ])
@@ -1502,7 +1602,6 @@ def kkt_qr(G, dims, A):
 
     Gs = matrix(0.0, (cdim, n))
     tauG = matrix(0.0, (n-p,1))
-    g = matrix(0.0, (cdim, 1))
     u = matrix(0.0, (cdim_pckd, 1))
     vv = matrix(0.0, (n,1))
     w = matrix(0.0, (cdim_pckd, 1))
@@ -1512,9 +1611,7 @@ def kkt_qr(G, dims, A):
         # Gs = W^{-T}*G, in packed storage.
         Gs[:,:] = G
         scale(Gs, W, trans = 'T', inverse = 'I')
-        for k in xrange(n):
-            g[:] = Gs[:, k]
-            pack(g, Gs, dims, offsety = k*Gs.size[0])
+        pack2(Gs, dims)
  
         # Gs := [ Gs1, Gs2 ] 
         #     = Gs * [ Q1, Q2 ]
@@ -1522,7 +1619,7 @@ def kkt_qr(G, dims, A):
 
         # QR factorization Gs2 := [ Q3, Q4 ] * [ R3; 0 ] 
         lapack.geqrf(Gs, tauG, n = n-p, m = cdim_pckd, offsetA = 
-                Gs.size[0]*p)
+            Gs.size[0]*p)
 
         def solve(x, y, z):
 
