@@ -4,7 +4,7 @@ CVXOPT interface for MOSEK 5.0
 
 # Copyright 2004-2008 J. Dahl and L. Vandenberghe.
 # 
-# This file is part of CVXOPT version 0.9.3.
+# This file is part of CVXOPT version 1.0.
 #
 # CVXOPT is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -32,7 +32,7 @@ inf = 0.0  # numeric value doesn't matter
 
 options = {}
 
-def solvelp(c, G, h, A=None, b=None):
+def lp(c, G, h, A=None, b=None):
     """
     Solves a pair of primal and dual LPs 
 
@@ -43,7 +43,7 @@ def solvelp(c, G, h, A=None, b=None):
                     
     using MOSEK 5.0.
 
-    (solsta, x, y, z) = solvelp(c, G, h, A=None, b=None).
+    (solsta, x, y, z) = lp(c, G, h, A=None, b=None).
 
     Input arguments 
 
@@ -185,7 +185,6 @@ def solvelp(c, G, h, A=None, b=None):
     else:
         return (solsta[0], x, z, y)
 
-
 def conelp(c, G, h, dims = None):
     """
     Solves a pair of primal and dual SOCPs
@@ -244,7 +243,7 @@ def conelp(c, G, h, dims = None):
     """
 
     if dims is None: 
-        (solsta, x, y, z) = solvelp(c, G, h)
+        (solsta, x, y, z) = lp(c, G, h)
         return (solsta, x, z, None)
 
     try:
@@ -538,7 +537,7 @@ def socp(c, Gl = None, hl = None, Gq = None, hq = None):
         return (solsta[0], x, zl, zq)
 
 
-def solveqp(P, q, G=None, h=None, A=None, b=None):
+def qp(P, q, G=None, h=None, A=None, b=None):
     """
     Solves a quadratic program
 
@@ -548,7 +547,7 @@ def solveqp(P, q, G=None, h=None, A=None, b=None):
                     
     using MOSEK 5.0.
 
-    (solsta, x, z, y) = solveqp(P, q, G=None, h=None, A=None, b=None)
+    (solsta, x, z, y) = qp(P, q, G=None, h=None, A=None, b=None)
 
     Return values
 
@@ -702,5 +701,172 @@ def solveqp(P, q, G=None, h=None, A=None, b=None):
         return (solsta[0], x, z, y)
 
 
+def ilp(c, G, h, A=None, b=None, I=None):
+    """
+    Solves the mixed integer LP
+
+        minimize    c'*x       
+        subject to  G*x + s = h
+                    A*x = b    
+                    s >= 0
+                    xi integer, forall i in I
+                    
+    using MOSEK 5.0.
+
+    (solsta, x) = ilp(c, G, h, A=None, b=None, I=None).
+
+    Input arguments 
+
+        G is m x n, h is m x 1, A is p x n, b is p x 1.  G and A must be 
+        dense or sparse 'd' matrices.   h and b are dense 'd' matrices 
+        with one column.  The default values for A and b are empty 
+        matrices with zero rows.
+
+        I is a Python set with indices of integer elements of x.  By 
+        default all elements in x are constrained to be integer, i.e.,
+        the default value of I is I = set(range(n))
+
+        Dual variables are not returned for MOSEK.
+
+    Return values
+
+        solsta   the solution status.
+                 If solsta is solsta.integer_optimal,
+                   then x contains the solution.
+                 If solsta is solsta.unknown,
+                   then x is None
+
+                 Other return values for solsta include:  
+                   mosek.solsta.near_integer_optimal
+                 in which case the x value may not be well-defined,
+                 c.f., the MOSEK documentation.
+        
+        x        the solution
+
+    Options are passed to MOSEK solvers via the mosek.options
+    dictionary, e.g., the following turns off output from 
+    the MOSEK solvers
+    
+    >>> mosek.options = {iparam.log:0} 
+    
+    see chapter 14 of the MOSEK Python API manual.                    
+    """
+    if type(c) is not matrix or c.typecode != 'd' or c.size[1] != 1: 
+        raise TypeError, "'c' must be a dense column matrix"
+    n = c.size[0]
+    if n < 1: raise ValueError, "number of variables must be at least 1"
+
+    if (type(G) is not matrix and type(G) is not spmatrix) or \
+        G.typecode != 'd' or G.size[1] != n:
+        raise TypeError, "'G' must be a dense or sparse 'd' matrix "\
+            "with %d columns" %n 
+    m = G.size[0]
+    if m is 0: raise ValueError, "m cannot be 0"
+
+    if type(h) is not matrix or h.typecode != 'd' or h.size != (m,1):
+        raise TypeError, "'h' must be a 'd' matrix of size (%d,1)" %m
+
+    if A is None:  A = spmatrix([], [], [], (0,n), 'd')
+    if (type(A) is not matrix and type(A) is not spmatrix) or \
+        A.typecode != 'd' or A.size[1] != n:
+        raise TypeError, "'A' must be a dense or sparse 'd' matrix "\
+            "with %d columns" %n
+    p = A.size[0]
+    if b is None: b = matrix(0.0, (0,1))
+    if type(b) is not matrix or b.typecode != 'd' or b.size != (p,1): 
+        raise TypeError, "'b' must be a dense matrix of size (%d,1)" %p
+ 
+    c   = array(c)        
+
+    if I is None: I = set(range(n))
+
+    if type(I) is not set: 
+        raise TypeError, "invalid argument for integer index set"
+
+    for i in I:
+        if type(i) is not int: 
+            raise TypeError, "invalid integer index set I"
+
+    if len(I) > 0 and min(I) < 0: raise IndexError, \
+            "negative element in integer index set I"
+    if len(I) > 0 and max(I) > n-1: raise IndexError, \
+            "maximum element in in integer index set I is larger than n-1"
+
+    bkc = m*[ msk.boundkey.up ] + p*[ msk.boundkey.fx ]
+    blc = m*[ -inf ] + [ bi for bi in b ]
+    buc = matrix([h, b])
+
+    bkx = n*[msk.boundkey.fr] 
+    blx = n*[ -inf ] 
+    bux = n*[ +inf ]
+
+    colptr, asub, acof = sparse([G,A]).CCS
+    aptrb, aptre = colptr[:-1], colptr[1:]
+
+    mskhandle = msk.mosek ()
+    env = mskhandle.Env ()
+    env.set_Stream (msk.streamtype.log, streamprinter)
+        
+    env.init () 
+    task = env.Task(0,0) 
+    task.set_Stream (msk.streamtype.log, streamprinter) 
+
+    # set MOSEK options 
+    for (param, val) in options.items():
+        if str(param)[:6] == "iparam":
+            task.putintparam(param, val)
+        elif str(param)[:6] == "dparam":
+            task.putdouparam(param, val)
+        elif str(param)[:6] == "sparam":
+            task.putstrparam(param, val)
+        else:
+            raise ValueError, "invalid MOSEK parameter: "+str(param)
+    
+    task.inputdata (m+p, # number of constraints
+                    n,   # number of variables
+                    array(c), # linear objective coefficients  
+                    0.0, # objective fixed value  
+                    array(aptrb), 
+                    array(aptre), 
+                    array(asub),
+                    array(acof), 
+                    array(bkc),
+                    array(blc),
+                    array(buc), 
+                    array(bkx),
+                    array(blx),
+                    array(bux)) 
+
+    task.putobjsense(msk.objsense.minimize)
+
+    # Define integer variables 
+    if len(I) > 0:
+        task.putvartypelist(array(I), 
+                            array(len(I)*[ msk.variabletype.type_int ])) 
+
+    task.putintparam (msk.iparam.mio_mode, msk.miomode.satisfied) 
+
+    task.optimize()
+
+    task.solutionsummary (msk.streamtype.msg); 
+
+    prosta, solsta = list(), list()
+    if len(I) > 0:
+        task.getsolutionstatus(msk.soltype.itg, prosta, solsta)
+    else:
+        task.getsolutionstatus(msk.soltype.bas, prosta, solsta)
+        
+    x = zeros(n, Float)
+    if len(I) > 0:
+        task.getsolutionslice(msk.soltype.itg, msk.solitem.xx, 0, n, x) 
+    else:
+        task.getsolutionslice(msk.soltype.bas, msk.solitem.xx, 0, n, x) 
+    x = matrix(x)
+
+    if (solsta[0] is msk.solsta.unknown):
+        return (solsta[0], None)
+    else:
+        return (solsta[0], x)
+
 
             
