@@ -148,11 +148,16 @@ amd = Extension('amd',
     sources = [ 'C/amd.c' ] + [ 'C/SuiteSparse/AMD/Source/' + s for s in
         listdir('C/SuiteSparse/AMD/Source') if s[-2:] == '.c' ])
 
-extmods += [base, blas, lapack, umfpack, cholmod, amd] 
+misc_solvers = Extension('misc_solvers', libraries = ['lapack', 'blas'],
+    library_dirs = [ ATLAS_LIB_DIR ],
+    define_macros = MACROS,
+    sources = ['C/misc_solvers.c'] )
+
+extmods += [base, blas, lapack, umfpack, cholmod, amd, misc_solvers] 
 
 setup (name = 'cvxopt', 
     description = 'Convex optimization package',
-    version = '0.9.3', 
+    version = '1.0', 
     long_description = '''
 CVXOPT is a free software package for convex optimization based on the 
 Python programming language. It can be used with the interactive Python 
